/*********************************************************
* HotProp Control
* Version: 1.8
* Date: September 1, 2003
* Autor: Michal Mecinski
* E-mail: mimec@mimec.w.pl
* WWW: http://www.mimec.w.pl
*
* Copyright (C) 2002-03 by Michal Mecinski
*********************************************************/

#include "stdafx.h"
#include "HPPopList.h"
#include "HotPropCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


CHPPopList::CHPPopList(CRect& rcRect, int nValue, BOOL bKey, int nItems,
	CFont* pFont, CWnd* pParent)
{
	LPCSTR lpszClass = AfxRegisterWndClass(0,
		LoadCursor(NULL, IDC_ARROW), NULL, NULL);

	int nBottom = rcRect.bottom;
	int nTop = rcRect.top;
	int nSize = nItems * 18 + 6;

	CRect rcWork;
	SystemParametersInfo(SPI_GETWORKAREA, 0, &rcWork, 0);

	BOOL bUnfold = TRUE;
#if (WINVER >= 0x0500)
	SystemParametersInfo(SPI_GETCOMBOBOXANIMATION, 0, &bUnfold, 0);
#endif

	int nInit = bUnfold ? 18 + 6 : nSize;

	// calculate best position and orientation
	if (nBottom + nSize <= rcWork.bottom)
	{
		rcRect.top = nBottom;
		rcRect.bottom = nBottom + nInit;
		m_bTop = FALSE;
	}
	else if (nTop - nSize >= rcWork.top)
	{
		rcRect.bottom = nTop;
		rcRect.top = nTop - nInit;
		m_bTop = TRUE;
	}
	else if (nTop - rcWork.top < rcWork.bottom - nBottom)
	{
		rcRect.top = rcWork.bottom - nSize;
		rcRect.bottom = rcRect.top + nInit;
		m_bTop = FALSE;
	}
	else
	{
		rcRect.bottom = rcWork.top + nSize;
		rcRect.top = rcRect.bottom - nInit;
		m_bTop = TRUE;
	}

	// use syncactive to prevent the main window from being deactivated
	Create(lpszClass, "", WS_POPUP|WS_VISIBLE|MFS_SYNCACTIVE, rcRect, pParent);

	m_pParent = pParent;
	m_pFont = pFont;

	m_nItems = nItems;
	m_nValue = nValue;
	m_nHot = bKey ? nValue : -1;	// hilite the current item if called from keyboard
	m_nReturn = -1;
	m_bCapture = FALSE;

	if (bUnfold && m_nItems>1)
	{
		m_nHeight = 1;	// start unfolding
		SetTimer(1, 120 / m_nItems, NULL);
	}
	else
		m_nHeight = m_nItems;

#ifdef HP_USE_LUNA_XP
	OSVERSIONINFO osvi;
	memset(&osvi, 0, sizeof(OSVERSIONINFO));
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx(&osvi);	// detect Windows XP
	m_bLunaXP = osvi.dwMajorVersion==5 && osvi.dwMinorVersion==1;
#else
 #ifdef XP_USE_LUNA_ALWAYS
	m_bLunaXP = TRUE;
 #else
	m_bLunaXP = FALSE;
 #endif
#endif
}


BEGIN_MESSAGE_MAP(CHPPopList, CMiniFrameWnd)
	ON_WM_KILLFOCUS()
	ON_WM_PAINT()
	ON_WM_KEYDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_CAPTURECHANGED()
	ON_WM_TIMER()
END_MESSAGE_MAP()


void CHPPopList::OnKillFocus(CWnd* pNewWnd) 
{
	m_pParent->SendMessage(HPPM_ENDPOP, m_nReturn);
	DestroyWindow();
}

void CHPPopList::OnPaint() 
{
	CPaintDC dc(this);

	CRect rcClient;
	GetClientRect(&rcClient);

	COLORREF crBack, crHalf;
	
	if (m_bLunaXP)
	{
		crBack = GetSysColor(COLOR_MENU);
		crHalf = CHotPropCtrl::LightenColor(GetSysColor(COLOR_HIGHLIGHT), 0.7);
	}
	else
	{
		crBack = GetSysColor(COLOR_BTNFACE);
		crHalf = CHotPropCtrl::LightenColor(crBack, 0.5);
	}

	dc.FillSolidRect(&rcClient, crBack);

	if (m_bLunaXP)
	{
		COLORREF crFrame = GetSysColor(COLOR_BTNSHADOW);
		dc.Draw3dRect(&rcClient, crFrame, crFrame);	// flat border
	}
	else
		dc.DrawEdge(&rcClient, BDR_RAISEDINNER, BF_RECT);

	dc.SetBkMode(TRANSPARENT);
	dc.SelectObject(m_pFont);

	// calculate visible part
	int nFirst = m_bTop ? 0 : m_nItems - m_nHeight;
	int nLast = m_bTop ? m_nHeight : m_nItems;

	for (int i=nFirst; i<nLast; i++)
	{
		CRect rcItem = rcClient;
		rcItem.top = 3 + (i-nFirst)*18;
		rcItem.bottom = rcItem.top + 18;

		rcItem.DeflateRect(3, 0);
		if (m_nHot==i && m_nHeight==m_nItems)
		{
			dc.FillSolidRect(&rcItem, crHalf);
			if (m_bLunaXP)
			{
				COLORREF crHigh = GetSysColor(COLOR_HIGHLIGHT);
				dc.Draw3dRect(&rcItem, crHigh, crHigh);
			}
			else
				dc.DrawEdge(&rcItem, BDR_SUNKENOUTER, BF_RECT);
		}

		rcItem.DeflateRect(14, 1, 1, 1);

		DrawInfo dinfo;
		dinfo.m_pDC = &dc;
		dinfo.m_rcRect = rcItem;
		m_pParent->SendMessage(HPPM_DRAWITEM, i, (LPARAM)&dinfo);

		if (m_nValue==i)	// draw a bullet to the left of current item
		{
			CFont font;
			font.CreatePointFont(100, "Marlett");
			dc.SelectObject(&font);

			if (m_nHot==i)
				dc.SetTextColor(GetSysColor(COLOR_HIGHLIGHT));

			rcItem.left = 3;
			dc.DrawText("i", 1, &rcItem, DT_SINGLELINE | DT_VCENTER);

			if (m_nHot==i)
				dc.SetTextColor(GetSysColor(COLOR_WINDOWTEXT));
			dc.SelectObject(m_pFont);
		}
	}
}


void CHPPopList::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	switch (nChar)
	{
	case VK_RETURN:
		m_nReturn = m_nHot;
	case VK_ESCAPE:
		AfxGetMainWnd()->SetFocus();	// kill focus to destroy the control
		break;

	case VK_UP:	// hilite previous item
		InvalidateItem(m_nHot);
		m_nHot--;
		if (m_nHot<0)
			m_nHot = m_nItems-1;
		InvalidateItem(m_nHot);
		break;

	case VK_DOWN:	// hilite next item
		InvalidateItem(m_nHot);
		m_nHot++;
		if (m_nHot>=m_nItems)
			m_nHot = 0;
		InvalidateItem(m_nHot);
		break;
	}
}


void CHPPopList::OnMouseMove(UINT nFlags, CPoint point) 
{
	CRect rcClient;
	GetClientRect(&rcClient);
	rcClient.DeflateRect(3, 3);

	if (!m_bCapture)	// similar to the CHotPropCtrl
	{
		SetCapture();
		m_bCapture = TRUE;
	}
	else
	{
		CPoint ptScr = point;
		ClientToScreen(&ptScr);
		if (WindowFromPoint(ptScr)!=this)
		{
			ReleaseCapture();
			return;
		}
	}

	int nItem = -1;

	if (rcClient.PtInRect(point))
		nItem = (point.y - 3)/18;

	if (nItem!=m_nHot)
	{
		InvalidateItem(m_nHot);
		InvalidateItem(nItem);
		m_nHot = nItem;
	}
}


void CHPPopList::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if (m_nHot>=0)
	{
		m_nReturn = m_nHot;	// accept choice
		AfxGetMainWnd()->SetFocus();
	}
}


void CHPPopList::InvalidateItem(int nPos)
{
	if (nPos>=0)
	{
		CRect rcClient;
		GetClientRect(&rcClient);
		rcClient.top = 3 + nPos*18;
		rcClient.bottom = rcClient.top + 18;
		InvalidateRect(&rcClient, FALSE);
	}
}

void CHPPopList::OnCaptureChanged(CWnd *pWnd) 
{
	m_bCapture = FALSE;

	if (m_nHot>=0)
	{
		InvalidateItem(m_nHot);
		m_nHot = -1;
	}
}

void CHPPopList::OnTimer(UINT nIDEvent) 
{
	if (m_nHeight >= m_nItems)	// unfolding done
	{
		if (m_nHot>=0)
			InvalidateItem(m_nHot);
		KillTimer(1);
		return;
	}
	m_nHeight++;

	CRect rcRect;	// expand window
	GetWindowRect(&rcRect);
	if (m_bTop)
		rcRect.top -= 18;
	else
		rcRect.bottom += 18;
	MoveWindow(&rcRect, FALSE);

	// scroll and draw the new item
	rcRect -= rcRect.TopLeft();
	if (m_bTop)
	{
		ScrollWindow(0, -18);
		rcRect.top = rcRect.bottom - 20;
	}
	else
	{
		ScrollWindow(0, 18);
		rcRect.bottom = 20;
	}
	InvalidateRect(&rcRect, FALSE);
}


